@@ -1,15 +1,18 @@
 """
 Libreria per fare grafici di fit e tabelle LaTeX a partire da dati raccolti in un file Excel
 TODO: graph log
+TODO: fit ricorsivo per errori sulla x con ciclo che termina quando non cambia più la soluzione oppure usare algoritmi esistenti che includono errore su x
+
 """
 __docformat__="numpy"
 
 import numpy as np
 from matplotlib import pyplot as plt
 from scipy.optimize import curve_fit
+from scipy.odr import *
 import pandas as pd
 from pint import UnitRegistry
-import re
+import re 
 
 def _tex_escape(text):
     """
@@ -44,6 +47,7 @@ def _converti_latex(text):
     return new_text
 
 def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_latex,
+    sigma_x=None,
     split_graphs=False,
     titoli=["",""],
     xlabel=None,
@@ -105,6 +109,11 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     titoli_p_latex : list
         elenco dei nomi dei parametri ottimali che vengono ricavati dal fit
         verranno usati come contenuto della cella del titolo di una tabella latex contenente i valori numerici dei parametri
+    
+    sigma_x: str, optional
+        contine il valore dell'header della colonna del foglio excel da cui prendere i dati
+        (deve contenere l'unità di misura nel formato: "[unità di misura]") (default: None)
+        se un valore viene specificato viene usato un metodo di fit con Orthogonal Distance Regression, altrimenti si usa scypi.curvefit()
 
     split_graphs: bool, optional
         se True la funzione genera due file separati per il grafico dei residui e il grafico di fit (default: False)
@@ -208,6 +217,12 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         if result:
             print(result)
     """
+    #definizione di una funzione della forma utile all'utilizzo di ODR
+    #TODO: vedere se è veramente necessario
+    def f_fit_odr (Beta, variabile):
+        return f_fit(variabile, *Beta)
+
+
 
     #verifica parametri
     if "[" not in variabile_x or "]" not in variabile_x: 
@@ -216,6 +231,9 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         return "Manca unità di misura in y"
     if "[" not in sigma_y or "]" not in sigma_y:
         return "Manca unità di misura in dy"
+    if sigma_x:
+        if "[" not in sigma_x or "]" not in sigma_x:
+            return "Manca unità di misura in dx"    
     if not file_name:
         return "specificare nome file excel"
     if not file_result:
@@ -227,12 +245,23 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     t=[e for e in excel[variabile_x].tolist() if e == e] #NaN != NaN quindi il test impedisce di scrivere i valori nulli
     y=[e for e in excel[variabile_y].tolist() if e == e]
     dy=[e for e in excel[sigma_y].tolist() if e == e]
+    if sigma_x:
+        dx=[e for e in excel[sigma_x].tolist() if e == e]
 
     #fit
     popt, pcov = curve_fit(f_fit,t,y,sigma=dy)
+
+    if sigma_x:
+
+        linear_model = Model(f_fit_odr)
+        data = RealData(t,y,dx,dy)
+        odr = ODR(data,linear_model,beta0=popt)
+        output = odr.run()
+        popt = output.beta
+
     chi=sum(np.power((y-f_fit(np.array(t),*popt)), 2)/np.power(dy,2))
     gradi_di_liberta=len(t)-len(popt)
-    chi_rid=chi/gradi_di_liberta
+    chi_rid=chi/gradi_di_liberta    
 
     #valutazione grafico
     sopra_fit=0
